import { View, Text, Image } from '@tarojs/components'
import MCheckbox from '@/components/checkbox'
import Counter from '@/components/counter'
import { FC, forwardRef, memo } from 'react'
import classnames from 'classnames'
import styles from './index.module.scss'
import { debounce } from '@/common/util'
import { formatImgUrl, formatPriceDiv } from '@/common/format'
import { EnumSaleMode } from '@/common/Enumerate'
import { selectList } from '../../config'
import { AdjestShoppingCartApi } from '@/api/shopping/index'
import { Goods, ShoppingDispatchType, ShoppingStateContextValue, useShoppingDispatch, useShoppingState } from '../../context'
import { ShoppingStore } from '../../context/shoppingStore'

type PropsType = {
  state?: {
    multipleSelection: Goods[]
    Observer: ShoppingStore
  }
  purchaserId: number
  itemData: Record<string, any> & object
  orderType: EnumSaleMode
}

let ColorKindItem: FC<PropsType> = props => {
  const { state, purchaserId, itemData, orderType = EnumSaleMode.Bulk } = props
  console.log('Rerender component: ColorKindItem', itemData.id)
  const dispatch = useShoppingDispatch()
  // console.log('checked==>', checked)

  //格式化金额
  const formatPrice = (price: number) => {
    return Number(formatPriceDiv(price))
  }

  //格式化数量
  const formatCount = itemData => {
    return itemData.sale_mode == EnumSaleMode.Bulk ? itemData.roll : itemData.length / 100
  }

  //格式化单位
  const formatUnit = itemData => {
    return itemData.sale_mode == EnumSaleMode.Bulk ? '条' : '米'
  }

  const handleSelect = () => {
    dispatch({
      type: ShoppingDispatchType.UPDATE_CHANGED_CHECKBOX,
      data: {
        purchaserId: purchaserId,
        multipleSelection: { ...state?.multipleSelection, [itemData.id]: itemData },
      },
    })
  }
  const handleClose = () => {
    const temp = state?.multipleSelection
    delete temp?.[itemData.id]
    dispatch({
      type: ShoppingDispatchType.UPDATE_CHANGED_CHECKBOX,
      data: {
        purchaserId: purchaserId,
        multipleSelection: temp,
      },
    })
  }
  const { fetchData } = AdjestShoppingCartApi()

  // 调整条数/米数的接口  并在调整完成后重新请求整个购物车页面
  const getInputValue = debounce(async (num, itemData) => {
    const targetColor: Record<string, any> = {
      product_color_id: itemData.product_color_id,
      roll: 0,
      length: 0,
    }
    if (itemData.sale_mode === EnumSaleMode.Bulk) {
      itemData.roll = num
      targetColor.roll = num
    } else {
      itemData.length = num
      targetColor.length = num
    }
    const res = await fetchData({
      color_list: [targetColor],
      purchaser_id: purchaserId,
      sale_mode: itemData.sale_mode,
      sale_offset: itemData.sale_offset,
    })
    if (res.success) {
      state?.Observer?.notify(purchaserId)
    }
  }, 400)

  return (
    <MCheckbox
      status={state?.multipleSelection?.hasOwnProperty(itemData.id) || false}
      onSelect={handleSelect}
      onClose={handleClose}
      customClassName={classnames(styles.checkbox, state?.multipleSelection?.hasOwnProperty(itemData.id) ? styles.selected : '')}
      customTextClass={styles.colorKindItem}>
      <View className={styles['colorKindItem__left']}>
        <Image className={styles['colorKindItem__left--image']} mode='aspectFill' src={formatImgUrl(itemData.product_color_texture_url)}></Image>
      </View>
      <View className={styles['colorKindItem__center']}>
        <Text className={styles['colorKindItem__center--title']}>
          {itemData.product_code}# {itemData.product_name}
        </Text>
        <View className={styles['colorKindItem__center--detail']}>
          <Text className={styles['colorKindItem__center--ID']}>{itemData.product_color_code}</Text>
          <Text className={styles['colorKindItem__center--description']}>{itemData.product_color_name}</Text>
        </View>
      </View>
      <View className={styles['colorKindItem__right']}>
        <View className={styles['colorKindItem__right--price']}>￥ {formatPrice(itemData.sale_price)}/kg</View>
        <View className={styles['colorKindItem__right--counter']}>
          <Counter
            onBlue={e => getInputValue(e, itemData)}
            defaultNum={formatCount(itemData)}
            step={selectList[orderType].step}
            digits={selectList[orderType].digits}
            onClickBtn={e => getInputValue(e, itemData)}
            unit={formatUnit(itemData)}
            minNum={itemData.min_num}
            maxNum={itemData.max_num}
          />
        </View>
      </View>
    </MCheckbox>
  )
}
// State 分割组件 思路就是把 context 直接通过 props 的形式传给组件，这样的话就解决了 context 强制刷新 memo 的问题了
// 那么当 context 内的 value 被更新的时候，react 只会强制渲染 Wrapper
const withStateSlice = (comp, slice) => {
  const MemoComp = memo(comp)
  const Wrapper = (props, ref) => {
    const state = useShoppingState()
    return <MemoComp ref={ref} state={slice(state, props)} {...props} />
  }
  return memo(forwardRef(Wrapper))
}

ColorKindItem = withStateSlice(ColorKindItem, (state: ShoppingStateContextValue, props: PropsType) => {
  return {
    multipleSelection: state.colorStore[props.purchaserId]['multipleSelection'],
    Observer: state.Observer,
  }
})

export default ColorKindItem
